package com.chrismweb.box2dforflashgames
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.display.Sprite
	import Box2D.Dynamics.*;
	import Box2D.Collision.*;
	import Box2D.Collision.Shapes.*;
	import Box2D.Common.Math.*;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	
	
	/**
	 * ...
	 * #Chp2.2 - Totem Level Setup
	 * @book by Emanuele Feronato
	 * http://www.emanueleferonato.com/
	 * Get it now @ http://www.packtpub.com/box2d-for-flash-games/book
	 */
	public class Main extends Sprite 
	{
		private var world:b2World;
		private var worldScale:Number=30;
		
		public function Main():void 
		{
			if (stage) 
				Init();
			else 
				addEventListener(Event.ADDED_TO_STAGE, Init);
		}
		
		private function Init(e:Event = null):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, Init);
			// entry point

			var gravity:b2Vec2 = new b2Vec2(0, 5);
			var sleep:Boolean = true;
			world = new b2World(gravity, sleep); //need to make sure you removed 'var', otherwise it will only initialize it as a local variable
			
			Setup();
			
			
			
			
			//stuff needed to reset example
			var reset_txt:TextField = new TextField();			
			var format:TextFormat = new TextFormat();
			format.color = 0xFFFFFFFF;
			format.align = TextFormatAlign.CENTER;
			format.size = 20;
			format.font = "Arial";
			
			reset_txt.width = 100;
			reset_txt.height = 30;
			reset_txt.selectable = false;
			reset_txt.text = "Reset";
			reset_txt.textColor = 0xFFFFFFFF;
			reset_txt.backgroundColor = 0xFF0D3451;
			reset_txt.opaqueBackground = reset_txt.backgroundColor;
			reset_txt.border = true;
			reset_txt.borderColor = 0xFFFFFFFF;
			reset_txt.defaultTextFormat = format;
			reset_txt.setTextFormat(format);
			addChild(reset_txt);
			reset_txt.addEventListener(MouseEvent.CLICK, function(e:MouseEvent):void { ResetWorld() } );
			
			
			
			DebugDraw();
			
			addEventListener(Event.ENTER_FRAME,UpdateWorld);
		}
		
		
		private function ResetWorld():void
		{
			trace("Resetting world");
			for (var body:b2Body=world.GetBodyList();body!=null; body=body.GetNext())
				world.DestroyBody(body);
				
			Setup();
		}
		private function Setup():void
		{
			Brick(275,435,30,30);
			Brick(365,435,30,30);
			Brick(320,405,120,30);
			Brick(320,375,60,30);
			Brick(305,345,90,30);
			Brick(320,300,120,60);
			
			Idol(320, 242);
			
			Floor();
		}
		
		
		private function DebugDraw():void 
		{
			//for enabling debug draw
			var debugSprite:Sprite = new Sprite();
			addChild(debugSprite);
			
			var debugDraw:b2DebugDraw = new b2DebugDraw();
			debugDraw.SetSprite(debugSprite);
			debugDraw.SetDrawScale(worldScale);
			debugDraw.SetFlags(b2DebugDraw.e_shapeBit);
			debugDraw.SetFillAlpha(0.5);
			world.SetDebugDraw(debugDraw);
		}
		
		
		private function UpdateWorld(e:Event):void 
		{
			var timeStep:Number = 1 / 30;
			var velIterations:int=10;
			var posIterations:int=10;
			world.Step(timeStep, velIterations, posIterations);
			world.ClearForces();
			world.DrawDebugData();
		}
		private function Brick(pX:int, pY:int, w:Number, h:Number):void 
		{
			
			var bodyDef:b2BodyDef=new b2BodyDef();
			bodyDef.position.Set(PixToMeter(pX),PixToMeter(pY));
			 bodyDef.type=b2Body.b2_dynamicBody; //ctrl+q to comment/uncomment quickly in FlashDevelop
			var polygonShape:b2PolygonShape=new b2PolygonShape();
			polygonShape.SetAsBox(PixToMeter(w/2),PixToMeter(h/2));
			var fixtureDef:b2FixtureDef=new b2FixtureDef();
			fixtureDef.shape=polygonShape;
			fixtureDef.density=2;
			fixtureDef.restitution=0.4;
			fixtureDef.friction=0.5;
			var thebrick:b2Body=world.CreateBody(bodyDef);
			thebrick.CreateFixture(fixtureDef);
		}
		
		private function Idol(pX:Number, pY:Number):void 
		{
			//create the initial axis oriented box
			var bodyDef:b2BodyDef=new b2BodyDef();
			bodyDef.position.Set(PixToMeter(pX), PixToMeter(pY));
			bodyDef.type = b2Body.b2_dynamicBody;
			
			var polygonShape:b2PolygonShape=new b2PolygonShape();
			polygonShape.SetAsBox(PixToMeter(5), PixToMeter(20));
			
			var fixtureDef:b2FixtureDef=new b2FixtureDef();
			fixtureDef.shape=polygonShape;
			fixtureDef.density=1;
			fixtureDef.restitution=0.4;
			fixtureDef.friction=0.5;
			var theIdol:b2Body=world.CreateBody(bodyDef);
			theIdol.CreateFixture(fixtureDef);
			
			//create the 2 45deg angle boxes:
			var bW:Number=PixToMeter(5);
			var bH:Number=PixToMeter(20);
			var boxPos:b2Vec2 = new b2Vec2(0, PixToMeter(10)); //trhis is actually positioned relative to the position of the first box!
			
			var boxAngle:Number = -Math.PI / 4; //first the counter-clockwise box PI=3.14rad=180deg / 4= 45
			polygonShape.SetAsOrientedBox(bW,bH,boxPos,boxAngle);
			fixtureDef.shape=polygonShape;
			theIdol.CreateFixture(fixtureDef);
			
			boxAngle=Math.PI/4;
			polygonShape.SetAsOrientedBox(bW,bH,boxPos,boxAngle);
			fixtureDef.shape=polygonShape;
			theIdol.CreateFixture(fixtureDef);
			
			//now actually create the last box as a polygon by actually specifying individual vertices! Convex and in clockwise order ONLY
			var vertices:Vector.<b2Vec2>=new Vector.<b2Vec2>();
			vertices.push(new b2Vec2(PixToMeter(-15), PixToMeter(-25)));
			vertices.push(new b2Vec2(PixToMeter(0),PixToMeter(-40)));
			vertices.push(new b2Vec2(PixToMeter(15), PixToMeter(-25)));
			vertices.push(new b2Vec2(PixToMeter(0),PixToMeter(-10)));
			polygonShape.SetAsVector(vertices,vertices.length);
			fixtureDef.shape=polygonShape;
			theIdol.CreateFixture(fixtureDef);
		}
		
		private function Floor():void 
		{
			var bodyDef:b2BodyDef=new b2BodyDef();
			bodyDef.position.Set(PixToMeter(320),PixToMeter(465));
			var polygonShape:b2PolygonShape=new b2PolygonShape();
			polygonShape.SetAsBox(PixToMeter(320),PixToMeter(15));
			var fixtureDef:b2FixtureDef=new b2FixtureDef();
			fixtureDef.shape=polygonShape;
			fixtureDef.restitution=0.4;
			fixtureDef.friction=0.5;
			var theFloor:b2Body=world.CreateBody(bodyDef);
			theFloor.CreateFixture(fixtureDef);
		}
		private function PixToMeter(pixels:Number):Number
		{
			return pixels / worldScale;
		}
	}
	
}